#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <iterator>
#include "code.h"

double Sample::get_temp()
{
    return stod(this->temp);
}
double Sample::get_snow()
{
    return stod(this->snow);
}

std::string Sample::get_time()
{
    return this->time;
}

void read(char *filename, std::vector<Sample> &samples) 
{
    std::fstream newfile;
    newfile.open(filename);
    if (!newfile.is_open())
    {
        std::cerr << "Could not open file\n";
        return;
    }

    std::string line;
    while (std::getline(newfile, line))
    {
        std::istringstream ss(line);
        std::istream_iterator<std::string> begin(ss), end;
        std::vector<std::string> words(begin, end);

        int n = words.size() - 1;
        Sample s{words[0], words[1], words[2]};
        for (int i = 0; i < n; i++)
            s.time += "," + words[i + 1];
        samples.push_back(s);
    }
    newfile.close();
}

enum State
{
    SQ_R,
    SQ_RF,
    SQ_GF2,
    SQ_RS,
    SQ_O,
    SQ_GF1,
    SQ_GS2,
    SQ_GS1,
    SQ_G
};
static double snowMeltLow(double T_e) 
{
    double lambda_s = 0.06;
    double U_r = 0.13;
    double T_i = 22;
    double Dml = -100 * (lambda_s * T_e) / (U_r * T_i);
    return Dml;
}

static double snowMeltHigh(double T_e) 
{
    double lambda_s = 0.18;
    double U_r = 0.13;
    double T_i = 22;
    double Dmh = -100 * (lambda_s * T_e) / (U_r * T_i);
    return Dmh;
}
State getState(double temp, double snowDepth)
{
    if (temp <= -5.6)
    {
        if (snowDepth >= snowMeltHigh(temp))
        {
            return SQ_R;
        }
        if (snowDepth >= snowMeltLow(temp))
        {
            return SQ_RF;
        }
        else
        {
            return SQ_GF2;
        }
    }
    if (temp >= 0)
    {
        if (snowDepth == 0)
        {
            return SQ_G;
        }
        else
        {
            return SQ_GS2;
        }
    }
    else
    {
        if (snowDepth >= snowMeltHigh(temp))
        {
            return SQ_RS;
        }
        if (snowDepth <= snowMeltLow(temp))
        {
            return SQ_GF1;
        }
        else
        {
            return SQ_O;
        }
    }
}

void analyzeData(std::vector<Sample> &samples) 
{

    int sq_r = 0;
    int sq_rf = 0;
    int sq_gf2 = 0;
    int sq_rs = 0;
    int sq_o = 0;
    int sq_gf1 = 0;
    int sq_gs2 = 0;
    int sq_gs1 = 0;
    int sq_g = 0;
    for (size_t i = 0; i < samples.size(); i++)
    {
        switch (getState(samples.at(i).get_temp(), 
                samples.at(i).get_snow()))
        {

        case SQ_R:
            sq_r++;
            break;
        case SQ_RF:
            sq_rf++;
            break;
        case SQ_GF2:
            sq_gf2++;
            break;
        case SQ_RS:
            sq_rs++;
            break;
        case SQ_O:
            sq_o++;
            break;
        case SQ_GF1:
            sq_gf1++;
            break;
        case SQ_GS2:
            sq_gs2++;
            break;
        case SQ_GS1:
            sq_gs1++;
            break;
        case SQ_G:
            sq_g++;
            break;
        default:
            break;
        }
    }
    std::cout << "Result in numbers: " << '\n'
              << sq_r << "\t" << sq_rs << "\t" << sq_gs2 << "\n"
              << sq_rf << "\t" << sq_o << "\t" << sq_gs1 << "\n"
              << sq_gf2 << "\t" << sq_gf1 << "\t" << sq_g << "\n"
              << "\n";

    int all = sq_r + sq_rs + sq_gs2 + sq_rf 
            + sq_o + sq_gs1 + sq_gf2 + sq_gf1 + sq_g;
    double dsq_r = (double)sq_r * 100 / all;
    double dsq_rs = (double)sq_rs * 100 / all;
    double dsq_gs2 = (double)sq_gs2 * 100 / all;
    double dsq_rf = (double)sq_rf * 100 / all;
    double dsq_o = (double)sq_o * 100 / all;
    double dsq_gs1 = (double)sq_gs1 * 100 / all;
    double dsq_gf2 = (double)sq_gf2 * 100 / all;
    double dsq_gf1 = (double)sq_gf1 * 100 / all;
    double dsq_g = (double)sq_g * 100 / all;
    double vpro = dsq_r + dsq_rs * 0.3 + dsq_rf * 0.5 + dsq_o * 0.15;
    std::cout << "Result in percent: " << '\n'
              << dsq_r << "\t" << dsq_rs << "\t" << dsq_gs2 << "\n"
              << dsq_rf << "\t" << dsq_o << "\t" << dsq_gs1 << "\n"
              << dsq_gf2 << "\t" << dsq_gf1 << "\t" << dsq_g << "\n"
              << "\n";
    std::cout << "Risiko = " << vpro << '\n';
}

std::string get_year(std::string input)
{
    std::string year = input.substr(0, 4);
    return year;
}

std::string get_month(std::string input)
{
    std::string month = input.substr(5, 7);
    return month;
}

enum Quarter
{
    q1,
    q2,
    q3,
    q4,
};

Quarter get_quarter(std::string monthnumber)
{
    int month = stoi(monthnumber);
    if (month <= 3)
    {
        return q1;
    }
    if (month > 3 && month <= 6)
    {
        return q2;
    }
    if (month > 6 && month <= 9)
    {
        return q3;
    }
    else
    {
        return q4;
    }
}

void checkData(std::vector<Sample> &samples)
{
    std::vector<double> listQ1;
    std::vector<double> listQ4;
    std::cout << "Year:" << '\t' 
              << "Total:" << '\t' 
              << "Q1" << '\t' 
              << "Q2" << '\t' 
              << "Q3" << '\t' 
              << "Q4" << '\t' << '\t' 
              << "%Total" << '\t' 
              << "%Q1" << '\t' 
              << "%Q2" << '\t' 
              << "%Q3" << '\t' 
              << "%Q4" << '\n';
    for (int j = 1996; j < 2022; j++)
    {
        int counter = 0;
        int counterQ1 = 0;
        int counterQ2 = 0;
        int counterQ3 = 0;
        int counterQ4 = 0;
        for (size_t i = 0; i < samples.size(); i++)
        {
            int year = stoi(get_year(samples.at(i).get_time()));
            if (year == j)
            {
                counter++;
                int quarter = 
                get_quarter(get_month(samples.at(i).get_time()));
                switch (quarter)
                {
                case q1:
                    counterQ1++;
                    break;
                case q2:
                    counterQ2++;
                    break;
                case q3:
                    counterQ3++;
                    break;
                case q4:
                    counterQ4++;
                    break;
                default:
                    break;
                }
            }
        }
        double devQ1 = 2160;
        double devQ2 = 2184;
        double devQ3 = 2208;
        double devQ4 = devQ3;
        if (j % 4 == 0)
        {
            devQ1 = devQ2;
        }
        double devT = devQ1 + devQ2 + devQ3 + devQ4;
        double pTot = (double)counter / devT;
        double pQ1 = (double)counterQ1 / devQ1;
        double pQ2 = (double)counterQ2 / devQ2;
        double pQ3 = (double)counterQ3 / devQ3;
        double pQ4 = (double)counterQ4 / devQ4;
        listQ1.push_back(pQ1);
        listQ4.push_back(pQ4);
        if (counter != 0)
        {
            std::cout.precision(4);
            std::cout << j << '\t' 
                      << counter << '\t' 
                      << counterQ1 << '\t' 
                      << counterQ2 << '\t' 
                      << counterQ3 << '\t' 
                      << counterQ4 << '\t' << '\t' 
                      << pTot * 100 << '\t' 
                      << pQ1 * 100 << '\t' 
                      << pQ2 * 100 << '\t' 
                      << pQ3 * 100 << '\t' 
                      << pQ4 * 100 << '\n';
        }
    }
    double tpQ1 = 0;
    double tpQ4 = 0;
    for (int i = 0; i < (int)listQ1.size(); i++)
    {
        tpQ4 += listQ4[i];
    }
    for (int i = 0; i < (int)listQ4.size(); i++)
    {
        tpQ1 += listQ1[i];
    }
    std::cout << '\n'
              << "Datasikkerhet = " << (tpQ1 + tpQ4) / 2 << '\n'
              << '\n';
    listQ1.clear();
    listQ4.clear();
}
